﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Base.Models;
using Base.DataAccessLayer;
using Kendo.Mvc.Extensions;
using Kendo.Mvc.UI;
using System.Web.Routing;

namespace Base.BusinessLogicLayer.Controllers
{
    public class TaskController : Controller
    {

        BaseEntities dbContext = new BaseEntities();

        public ActionResult Index()
        {
            ViewBag.Message = "Welcome to ASP.NET MVC!";

            //Set pager
            var R = new PagerViewModel();

            //R.Input = true;         //Have an input box for the page number (default off)
            //R.Numeric = true;       //Have numeric pager (default off)
            //R.Info = true;          //Display the current range of items (default off)
            //R.PreviousNext = true;  //Display the 'first', 'previous', 'next' and 'last' buttons (default off)
            //R.Refresh = true;       //Display the 'refresh' button (default off)
            //R.PageSizes = true;     //Allow the user to change the page size (default off)
            R.PageSizeSelections = new int[] { 10, 20, 30, 40, 50 };

            return View(R);
            
            //return View();
        }

        [HttpPost]
        public ActionResult Index(PagerViewModel pager)
        {
            return View(pager);
        }

        public ActionResult GetTasks([DataSourceRequest] DataSourceRequest request)
        {
            return Json(GetTaskViewModels().ToDataSourceResult(request));
        }

        private IQueryable<TaskViewModel> GetTaskViewModels()
        {
            return dbContext.Task
                            .Select(
                            c => new TaskViewModel { TaskID = c.TaskID, TaskName = c.TaskName, TaskText = c.TaskText, CreateDate = c.CreateDate });
        }

        public ActionResult UpdateTask([DataSourceRequest] DataSourceRequest request, Task task)
        {
            var taskToUpdate = dbContext.Task.First(r => r.TaskID == task.TaskID);

            TryUpdateModel(taskToUpdate);

            dbContext.SaveChanges();

            return Json(GetTaskViewModels().ToDataSourceResult(request));
        }

        public ActionResult InsertTask([DataSourceRequest] DataSourceRequest request, Task taskToAdd)
        {
            if (ModelState.IsValid)
            {
                dbContext.Task.Add(taskToAdd);
                dbContext.SaveChanges();
            }

            return Json(new[] { taskToAdd }.ToDataSourceResult(request));
        }

        public ActionResult DeleteTask([DataSourceRequest] DataSourceRequest request, Task task)
        {
            var taskToDelete = dbContext.Task.First(r => r.TaskID == task.TaskID);

            if (taskToDelete != null)
            {
                dbContext.Task.Remove(taskToDelete);
                dbContext.SaveChanges();
            }

            return Json(new[] { taskToDelete }.ToDataSourceResult(request));
        }

    }

}
