﻿// Copyright © 2012 Matthew David Elgert - mdelgert@yahoo.com
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//  
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 
//   
// Project URL: https://kendoui.codeplex.com/
//    
// CodePlex Author: http://www.codeplex.com/site/users/view/mdelgert
// 
// Date: 11/6/2012
// 
  
using System;
using System.Data.Entity;
using System.Linq;
using Base.Models;

namespace Base.DataAccessLayer
{

    public class BaseEntities : DbContext
    {

        public DbSet<PersonAccountModel> PersonAccount { get; set; }
        public DbSet<Task> Task { get; set; }

        public DbSet<MiniProfilers> MiniProfilers { get; set; }

        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
            /// <summary>
            /// Code First pluralizes specify the table names - http://msdn.microsoft.com/en-us/magazine/hh126815.aspx
            /// Usage: modelBuilder.Entity<YourModel>().ToTable("DatabaseTableName", schemaName: "SchemaName"); modelBuilder.Entity<YourModel>().ToTable("DatabaseTableName"); //Default schema dbo
            /// </summary>
            
            modelBuilder.Entity<PersonAccountModel>().ToTable("PersonAccount");
            modelBuilder.Entity<Task>().ToTable("Task");

            //http://stackoverflow.com/questions/3504660/entity-framework-code-first-decimal-precision-and-scale
            modelBuilder.Entity<MiniProfilers>().Property(product => product.DurationMilliseconds).HasPrecision(7, 1);
            modelBuilder.Entity<MiniProfilers>().Property(product => product.DurationMillisecondsInSql).HasPrecision(7, 1);
            modelBuilder.Entity<MiniProfilers>().Property(product => product.TrivialDurationThresholdMilliseconds).HasPrecision(5, 1);
            modelBuilder.Entity<MiniProfilers>().ToTable("MiniProfilers");
        
        
        }

        public static void Setup()
        {
            ///<summary>
            ///Available Database Initializers -- http://www.codeguru.com/csharp/article.php/c19999/Understanding-Database-Initializers-in-Entity-Framework-Code-First.htm
            ///Entity Framework Code First comes with three basic database initializers 1. CreateDatabaseIfNotExists 2. DropCreateDatabaseWhenModelChanges 3. DropCreateDatabaseAlways
            ///</summary>
        
            //Database.SetInitializer(new CreateDatabaseIfNotExists<BaseEntities>()); //1. This is the default database initializer class used by Code First unless you specify some other class. As the name suggests, the CreateDatabaseIfNotExists class creates a database only if it doesn't exist. This initializer also comes in handy when you wish to avoid any accidental deletion of the database.
            //Database.SetInitializer(new DropCreateDatabaseAlways<BaseEntities>()); //2. This database initializer creates a database if it doesn't exist already. Additionally, if a database is already present but there is a mismatch between the model classes and table schema then it deletes the database and re-creates it. You will find this more useful during development and testing when models are changing often.
            //Database.SetInitializer(new DropCreateDatabaseAlways<BaseEntities>()); //3. The DropCreateDatabaseAlways class deletes and creates a database irrespective of whether it is already present or not. This way, with every run of the application you will be deleting and re-creating the database. You will find this initializer useful during testing when you want to run the application with a fresh set of data.

            //RunSQLScripts();

        }

        private static void RunSQLScripts()
        {
            ///<summary>
            ///Database setup scripts during initialization
            ///</summary>
            
            using (var context = new BaseEntities())
            {
                context.Database.ExecuteSqlCommand("CREATE NONCLUSTERED INDEX [IX_CreatedOn] ON dbo.LogApplication ([CreatedOn])");
            }

        }

    }

}
