﻿using System;
using System.Linq;
using System.Web.Mvc;
using KendoGridMvcCodeFirst.Models;
using KendoGridMvcCodeFirst.ViewModels;
using Kendo.Mvc.Extensions;
using Kendo.Mvc.UI;

namespace KendoGridMvcCodeFirst.Controllers
{
    public class TaskController : Controller
    {
        RepositoriesContext dbContext = new RepositoriesContext();

        public ActionResult Index()
        {
            ViewBag.Message = "Welcome to ASP.NET MVC!";
            return View();
        }

        public ActionResult GetTasks([DataSourceRequest] DataSourceRequest request)
        {
            return Json(GetTaskViewModels().ToDataSourceResult(request));
        }

        private IQueryable<TaskViewModel> GetTaskViewModels()
        {
            return dbContext.Task
                            .Select(
                            c => new TaskViewModel
                            { TaskID = c.TaskID, TaskText = c.TaskText, CreateDate = c.CreateDate});
        }

        public ActionResult UpdateTask([DataSourceRequest] DataSourceRequest request, Task task)
        {
            var taskToUpdate = dbContext.Task.First(r => r.TaskID == task.TaskID);

            TryUpdateModel(taskToUpdate);

            dbContext.SaveChanges();

            return Json(GetTaskViewModels().ToDataSourceResult(request));
        }

        public ActionResult InsertTask([DataSourceRequest] DataSourceRequest request, Task taskToAdd)
        {
            if (ModelState.IsValid)
            {
                dbContext.Task.Add(taskToAdd);
                dbContext.SaveChanges();
            }

            return Json(new[] { taskToAdd }.ToDataSourceResult(request));
        }

        public ActionResult DeleteTask([DataSourceRequest] DataSourceRequest request, Task task)
        {
            var taskToDelete = dbContext.Task.First(r => r.TaskID == task.TaskID);

            if (taskToDelete != null)
            {
                dbContext.Task.Remove(taskToDelete);
                dbContext.SaveChanges();
            }

            return Json(new[] { taskToDelete }.ToDataSourceResult(request));
        }

    }

}
